$Title Linearised Urban Water Small
$maxcol 125
$onsymlist
$onsymxref
$oninline
$ontext
*** Linear Urban Water Small ***
Outlined below is a multistage stochastic spatial and temporal programming model of an urban
water system. The model maximises Net Social Welfare (Marshallian Consumer plus producer
surplus).
The model includes investment options for desal, aquifers, recycling, tanks, and trade
$offtext
option mip=gurobi ;
option mip=gurobi ;

*option lp=minos ;
option decimals = 4 ;
option limcol = 0 ;
option limrow = 0 ;
option solslack = 1 ;
OPTION RESLIM = 100000 ;
OPTION ITERLIM = 100000000 ;
option solprint = on ;
option profile = 1 ;
option sysout=on ;

Parameter FixedD(Yr) ;
FixedD(Yr) = Gf(Yr) * 276 ;
*FixedD(Yr) = Gf(Yr) * 256 ;

* Variables and equations constituting the model
Free variables
  NSW  expected present value of net social welfare (marshallian C+P surplus)
  Cost expected present value of costs ;

Positive variables
  Qd(Yr,N,D,L) Linearised demand variable for aggregate demand at each node
  Qddam(Yr,N,M) Amount of water stored in the terminal year T
  Qsdam(Yr,N) Quantity supplied from dam in node
  Qstdam(Yr,N) Quantity of water stored in dam in each node
  Qsdesal1(Yr,N) Quantity supplied from existing desal (Kwinana) at each node
*  Qsdesal2(Yr,N) Quantity supplied from existing desal (Southern) at each node
  Qsaquif0(Yr,N) Quantity supplied from existing aquifers at each node
  Qsdesal(Yr,N) Quantity supplied from desal at each node
  Qsaquif(Yr,N) Quantity supplied from aquifers at each node
  Qsrecyc(Yr,N) Quantity supplied from recycling at each node
  Qstanks(Yr,N) Quantity supplied from tanks at each node
  Qstrade(Yr,N) Quantity supplied from trade at each node
  Qsaquif2 (Yr,N) Quantity supplied from large aquifers at each node ;
  ;

Binary variable
  Qdesalcap(Yr,N) additional desal capacity installed at each node
  Qdesalcap2(Yr) additional desal capacity installed at each node
  Qrecyccap(Yr,N) additional recycling (groundwater replenishment) capacity installed at each node
  Qrecyccap2(Yr) additional recycling (groundwater replenishment) capacity installed at each node
  Qaquif2cap(Yr,N) additionl large aquifer capacity installed at each node
  Qaquif2cap2(Yr) additionl large aquifer capacity installed at each node  ;

Positive variables
  Qtankscap(Yr,N) additional tank capacity installed at each node
  Qtankscap2(Yr) additional tank capacity installed at each node
  Qtradecap(Yr,N) additional trade capacity installed at each node
  Qtradecap2(Yr) additional trade capacity installed at each node
  Qaquifcap(Yr,N) additional aquifer capacity installed at each node
  Qaquifcap2(Yr) additional aquifer capacity installed at each node  ;


Equations

  aquifsupply(Yr,N) water supply from aquifers in each node
  aquifmaxcap(Yr,N) maximum aquifer capacity
  recycsupply(Yr,N) water supply from recycling in each node
  recycmaxcap(Yr,N) maximum recycling capacity
  tankssupply(Yr,N) water supply from tanks in each node
  tanksmaxcap(Yr,N) maximum tank capacity
  tradesupply(Yr,N) water supply from trade in each node
  trademaxcap(Yr,N) maximum trade capacity
  aquif2supply(Yr,N) water supply from large aquifers in each node
  aquif2maxcap(Yr,N) maximum large aquifer capacity  ;

Equations
  Objfn Objective function nsw max multi-stage
  Objfn2 Objective function nsw max two-stage
  ObjfnC Objective function cost min multi-stage
  ObjfnC2 Objective function cost min two-stage
  ConVexDem(Yr,N,D) constraint for convexity of linearised demand
  ConVexQstdam(Yr,N) constraint for convexity of terminal storage
  demand(Yr,N) balance of water demanded
  demandF(Yr,N) balance of water demanded with fixed demand target (cost min)
  demdamstortp(Yr,N) balance of the demand for dam water storage in the last year
  damsupply(Yr,N) water supply from dams
  dammaxstor(Yr,N) maximum water storage capacity of dams
  aquifavgsupply(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply1(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply2(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply3(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply4(Yr,N) average supply from aquifers for each scenario
  desalsupply(Yr,N) water supply from desal in each node
  desalsupply2(Yr,N) water supply from desal in each node 2
  desalmaxcap(Yr,N) maximum desal capacity
  desalmaxcap2 maximum desal capacity 2
  aquifsupply(Yr,N) water supply from aquifers in each node
  aquifsupply2(Yr,N) water supply from aquifers in each node 2
  aquifmaxcap(Yr,N) maximum aquifer capacity
  aquifmaxcap2 maximum aquifer capacity 2
  recycsupply(Yr,N) water supply from recycling in each node
  recycsupply2(Yr,N) water supply from recycling in each node 2
  recycmaxcap(Yr,N) maximum recycling capacity
  recycmaxcap2 maximum recycling capacity 2
  tankssupply(Yr,N) water supply from tanks in each node
  tankssupply2(Yr,N) water supply from tanks in each node 2
  tanksmaxcap(Yr,N) maximum tank capacity
  tanksmaxcap2 maximum tank capacity 2
  tradesupply(Yr,N) water supply from trade in each node
  tradesupply2(Yr,N) water supply from trade in each node 2
  trademaxcap(Yr,N) maximum trade capacity
  trademaxcap2 maximum trade capacity 2
  aquif2supply(Yr,N) water supply from large aquifers in each node
  aquif2supply2(Yr,N) water supply from large aquifers in each node 2
  aquif2maxcap(Yr,N) maximum large aquifer capacity
  aquif2maxcap2 maximum large aquifer capacity 2 ;

*  Objfn      Objective function to maximise net social welfare
Objfn..
  nsw =e=
    sum{YrNd(Yr,N),
      sum[(D,L),
        Area(Yr,N,D,L)*Qd(Yr,N,D,L)
      ]

      + sum[M$LastYrNd(Yr,N),
         AreaTS(Yr,N,M)*Qddam(Yr,N,M)
      ]

      - [CapExDesal(Yr,N)*Qdesalcap(Yr,N)]$DesalCapInv(Yr,N)
      - [OpExDesal(Yr,N)*Qsdesal(Yr,N)]$DesalProdnYrNd(Yr,N)

      - [CapExAquif(Yr,N)*Qaquifcap(Yr,N)]$AquifCapInv(Yr,N)
      - [OpExAquif(Yr,N)*Qsaquif(Yr,N)]$AquifProdnYrNd(Yr,N)

      - [CapExRecyc(Yr,N)*Qrecyccap(Yr,N)]$RecycCapInv(Yr,N)
      - [OpExRecyc(Yr,N)*Qsrecyc(Yr,N)]$RecycProdnYrNd(Yr,N)

      - [CapExTanks(Yr,N)*Qtankscap(Yr,N)]$TanksCapInv(Yr,N)
      - [OpExTanks(Yr,N)*Qstanks(Yr,N)]$TanksProdnYrNd(Yr,N)

      - [CapExTrade(Yr,N)*Qtradecap(Yr,N)]$TradeCapInv(Yr,N)
      - [OpExTrade(Yr,N)*Qstrade(Yr,N)]$TradeProdnYrNd(Yr,N)

      - [CapExAquif2(Yr,N)*Qaquif2cap(Yr,N)]$Aquif2CapInv(Yr,N)
      - [OpExAquif2(Yr,N)*Qsaquif2(Yr,N)]$Aquif2ProdnYrNd(Yr,N)

      - [OpExDam(Yr,N)*Qsdam(Yr,N)]
      - [OpExDesal1(Yr,N)*Qsdesal1(Yr,N)]
      - [OpExAquif0(Yr,N)*Qsaquif0(Yr,N)]
*      - [OpExDesal2(Yr,N)*Qsdesal2(Yr,N)]$Desal2ProdnYrNd(Yr,N)
    } ;

*  Objfn2      Objective function to maximise net social welfare two-stage
Objfn2..
  nsw =e=
    sum{YrNd(Yr,N),
      sum[(D,L),
        Area(Yr,N,D,L)*Qd(Yr,N,D,L)
      ]

      + sum[M$LastYrNd(Yr,N),
         AreaTS(Yr,N,M)*Qddam(Yr,N,M)
      ]

      - [CapExDesal(Yr,N)*Qdesalcap2(Yr)]$DesalCapInv(Yr,N)
      - [OpExDesal(Yr,N)*Qsdesal(Yr,N)]$DesalProdnYrNd(Yr,N)

      - [CapExAquif(Yr,N)*Qaquifcap2(Yr)]$AquifCapInv(Yr,N)
      - [OpExAquif(Yr,N)*Qsaquif(Yr,N)]$AquifProdnYrNd(Yr,N)

      - [CapExRecyc(Yr,N)*Qrecyccap2(Yr)]$RecycCapInv(Yr,N)
      - [OpExRecyc(Yr,N)*Qsrecyc(Yr,N)]$RecycProdnYrNd(Yr,N)

      - [CapExTanks(Yr,N)*Qtankscap2(Yr)]$TanksCapInv(Yr,N)
      - [OpExTanks(Yr,N)*Qstanks(Yr,N)]$TanksProdnYrNd(Yr,N)

      - [CapExTrade(Yr,N)*Qtradecap2(Yr)]$TradeCapInv(Yr,N)
      - [OpExTrade(Yr,N)*Qstrade(Yr,N)]$TradeProdnYrNd(Yr,N)

      - [CapExAquif2(Yr,N)*Qaquif2cap2(Yr)]$Aquif2CapInv(Yr,N)
      - [OpExAquif2(Yr,N)*Qsaquif2(Yr,N)]$Aquif2ProdnYrNd(Yr,N)

      - [OpExDam(Yr,N)*Qsdam(Yr,N)]
      - [OpExDesal1(Yr,N)*Qsdesal1(Yr,N)]
      - [OpExAquif0(Yr,N)*Qsaquif0(Yr,N)]
*      - [OpExDesal2(Yr,N)*Qsdesal2(Yr,N)]$Desal2ProdnYrNd(Yr,N)
    } ;


*  Objfn      Objective function to minimise cost multi-stage
ObjfnC..
  cost =e=
    sum{YrNd(Yr,N),
*      sum[(D,L),
*        Area(Yr,N,D,L)*Qd(Yr,N,D,L)
*      ]

*      + sum[M$LastYrNd(Yr,N),
*         AreaTS(Yr,N,M)*Qddam(Yr,N,M)
*      ]

       [CapExDesal(Yr,N)*Qdesalcap(Yr,N)]$DesalCapInv(Yr,N)
      + [OpExDesal(Yr,N)*Qsdesal(Yr,N)]$DesalProdnYrNd(Yr,N)

      + [CapExAquif(Yr,N)*Qaquifcap(Yr,N)]$AquifCapInv(Yr,N)
      + [OpExAquif(Yr,N)*Qsaquif(Yr,N)]$AquifProdnYrNd(Yr,N)

      + [CapExRecyc(Yr,N)*Qrecyccap(Yr,N)]$RecycCapInv(Yr,N)
      + [OpExRecyc(Yr,N)*Qsrecyc(Yr,N)]$RecycProdnYrNd(Yr,N)

      + [CapExTanks(Yr,N)*Qtankscap(Yr,N)]$TanksCapInv(Yr,N)
      + [OpExTanks(Yr,N)*Qstanks(Yr,N)]$TanksProdnYrNd(Yr,N)

      + [CapExTrade(Yr,N)*Qtradecap(Yr,N)]$TradeCapInv(Yr,N)
      + [OpExTrade(Yr,N)*Qstrade(Yr,N)]$TradeProdnYrNd(Yr,N)

      + [CapExAquif2(Yr,N)*Qaquif2cap(Yr,N)]$Aquif2CapInv(Yr,N)
      + [OpExAquif2(Yr,N)*Qsaquif2(Yr,N)]$Aquif2ProdnYrNd(Yr,N)

      + [OpExDam(Yr,N)*Qsdam(Yr,N)]
      + [OpExDesal1(Yr,N)*Qsdesal1(Yr,N)]
      + [OpExAquif0(Yr,N)*Qsaquif0(Yr,N)]
*      + [OpExDesal2(Yr,N)*Qsdesal2(Yr,N)]$Desal2ProdnYrNd(Yr,N)
    } ;

*  Objfn2      Objective function to minimise cost two-stage
ObjfnC2..
  cost =e=
    sum{YrNd(Yr,N),
*      sum[(D,L),
*        Area(Yr,N,D,L)*Qd(Yr,N,D,L)
*      ]

*      + sum[M$LastYrNd(Yr,N),
*         AreaTS(Yr,N,M)*Qddam(Yr,N,M)
*      ]

       [CapExDesal(Yr,N)*Qdesalcap2(Yr)]$DesalCapInv(Yr,N)
      + [OpExDesal(Yr,N)*Qsdesal(Yr,N)]$DesalProdnYrNd(Yr,N)

      + [CapExAquif(Yr,N)*Qaquifcap2(Yr)]$AquifCapInv(Yr,N)
      + [OpExAquif(Yr,N)*Qsaquif(Yr,N)]$AquifProdnYrNd(Yr,N)

      + [CapExRecyc(Yr,N)*Qrecyccap2(Yr)]$RecycCapInv(Yr,N)
      + [OpExRecyc(Yr,N)*Qsrecyc(Yr,N)]$RecycProdnYrNd(Yr,N)

      + [CapExTanks(Yr,N)*Qtankscap2(Yr)]$TanksCapInv(Yr,N)
      + [OpExTanks(Yr,N)*Qstanks(Yr,N)]$TanksProdnYrNd(Yr,N)

      + [CapExTrade(Yr,N)*Qtradecap2(Yr)]$TradeCapInv(Yr,N)
      + [OpExTrade(Yr,N)*Qstrade(Yr,N)]$TradeProdnYrNd(Yr,N)

      + [CapExAquif2(Yr,N)*Qaquif2cap2(Yr)]$Aquif2CapInv(Yr,N)
      + [OpExAquif2(Yr,N)*Qsaquif2(Yr,N)]$Aquif2ProdnYrNd(Yr,N)

      + [OpExDam(Yr,N)*Qsdam(Yr,N)]
      + [OpExDesal1(Yr,N)*Qsdesal1(Yr,N)]
      + [OpExAquif0(Yr,N)*Qsaquif0(Yr,N)]
*      + [OpExDesal2(Yr,N)*Qsdesal2(Yr,N)]$Desal2ProdnYrNd(Yr,N)
    } ;


*  ConVexDem(Yr,N) constraint for convexity of linearised demands
ConVexdem(YrNd(Yr,N),D)..
  sum(L,Qd(Yr,N,D,L)) =l= 1.0 ;

* demand(Yr,N) water demand balance
demand(YrNd(Yr,N))..
  sum[(D,L),Qty(Yr,N,D,L)*Qd(Yr,N,D,L)]
  - qsdam(Yr,N) - qsdesal1(Yr,N)
*  - qsdesal2(Yr,N)$Desal2ProdnYrNd(Yr,N)
  - qsaquif0(Yr,N)
  - qsdesal(Yr,N)$DesalProdnYrNd(Yr,N) - qsaquif(Yr,N)$AquifProdnYrNd(Yr,N)
  - qsrecyc(Yr,N)$RecycProdnYrNd(Yr,N) - qstanks(Yr,N)$TanksProdnYrNd(Yr,N)
  - qstrade(Yr,N)$TradeProdnYrNd(Yr,N)
  - qsaquif2(Yr,N)$aquif2ProdnYrNd(Yr,N)
  =l= 0 ;

* demand(Yr,N) water demand balance
demandF(YrNd(Yr,N))..
*  sum[(D,L),Qty(Yr,N,D,L)*Qd(Yr,N,D,L)]
  FixedD(Yr)
  - qsdam(Yr,N) - qsdesal1(Yr,N)
*  - qsdesal2(Yr,N)$Desal2ProdnYrNd(Yr,N)
  - qsaquif0(Yr,N)
  - qsdesal(Yr,N)$DesalProdnYrNd(Yr,N) - qsaquif(Yr,N)$AquifProdnYrNd(Yr,N)
  - qsrecyc(Yr,N)$RecycProdnYrNd(Yr,N) - qstanks(Yr,N)$TanksProdnYrNd(Yr,N)
  - qstrade(Yr,N)$TradeProdnYrNd(Yr,N)
  - qsaquif2(Yr,N)$aquif2ProdnYrNd(Yr,N)
  =l= 0 ;

* demdamstortp(Yr,N) balance of the demand for dam water storage in the terminal period
demdamstortp(LastYrNd(Yr,N))..
  sum[M,QtyTS(Yr,N,M)*Qddam(Yr,N,M)] - Qstdam(Yr,N) =l= 0 ;

* ConVexQstdam(Yr,N)constraint for convexity of terminal storage
ConVexQstdam(LastYrNd(Yr,N))..
  sum(M,Qddam(Yr,N,M)) =e= 1.0 ;

* damsupply(Yr,N) water supply from dam
damsupply(YrNd(Yr,N))..
  Qsdam(Yr,N) + Qstdam(Yr,N) - sum[(YrP,NP)$ChildParent(Yr,N,YrP,NP),(1-EvapLoss)*Qstdam(YrP,NP)]
  =l= InflowAtNode(Yr,N) ;

* dammaxstor(Yr,N) maximum water storage capacity of dam in any year
dammaxstor(YrNd(Yr,N))..
  Qstdam(Yr,N) =l= MaxS0 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply(lastyrnd(Yr,N))..
  sum[(Yrp,Np)$ScenMap(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] + 433 =l= [card(yr)+3]*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply1(YrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap1(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 5*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply2(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap2(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 4*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply3(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap3(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 3*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply4(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap4(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 2*120 ;

* desalsupply(Yr,N) maximum water supply from desal in any year
desalsupply(DesalProdnYrNd(Yr,N))..
  Qsdesal(Yr,N)-sum((YrP,NP)$DesalCapMap(Yr,N,YrP,NP),DesalUnitSize*Qdesalcap(YrP,NP)) =l= 0 ;
desalsupply2(DesalProdnYrNd(Yr,N))..
  Qsdesal(Yr,N)-sum((YrP,NP)$DesalCapMap(Yr,N,YrP,NP),DesalUnitSize*Qdesalcap2(YrP)) =l= 0 ;

* desalmaxcap upper bound on the total installed desal capacity
desalmaxcap(DesalCapLastYrNd(Yr,N))..
  sum((YrP,NP)$DesalCapMaxMap(Yr,N,YrP,NP),Qdesalcap(YrP,NP)) =l= desalicap ;
desalmaxcap2..
  sum( Yr , Qdesalcap2(Yr) ) =l= desalicap ;

* aquifsupply(Yr,N) maximum water supply from aquifers in any year
aquifsupply(AquifProdnYrNd(Yr,N))..
  Qsaquif(Yr,N)-sum((YrP,NP)$AquifCapMap(Yr,N,YrP,NP),AquifUnitSize*Qaquifcap(YrP,NP)) =l= 0 ;
aquifsupply2(AquifProdnYrNd(Yr,N))..
  Qsaquif(Yr,N)-sum((YrP,NP)$AquifCapMap(Yr,N,YrP,NP),AquifUnitSize*Qaquifcap2(YrP)) =l= 0 ;

* aquifmaxcap upper bound on the total installed aquifer capacity
aquifmaxcap(AquifCapLastYrNd(Yr,N))..
  sum((YrP,NP)$AquifCapMaxMap(Yr,N,YrP,NP),Qaquifcap(YrP,NP)) =l= aquificap ;
aquifmaxcap2..
  sum( Yr , Qaquifcap2(Yr) ) =l= aquificap ;

* recycsupply(Yr,N) maximum water supply from recycling in any year
recycsupply(RecycProdnYrNd(Yr,N))..
  Qsrecyc(Yr,N)-sum((YrP,NP)$RecycCapMap(Yr,N,YrP,NP),RecycUnitSize*Qrecyccap(YrP,NP)) =l= 0 ;
recycsupply2(RecycProdnYrNd(Yr,N))..
  Qsrecyc(Yr,N)-sum((YrP,NP)$RecycCapMap(Yr,N,YrP,NP),RecycUnitSize*Qrecyccap2(YrP)) =l= 0 ;

* recycmaxcap upper bound on the total installed recyc capacity
recycmaxcap(RecycCapLastYrNd(Yr,N))..
  sum((YrP,NP)$RecycCapMaxMap(Yr,N,YrP,NP),Qrecyccap(YrP,NP)) =l= recycicap ;
recycmaxcap2..
  sum( Yr , Qrecyccap2(Yr) ) =l= recycicap  ;

* tankssupply(Yr,N) maximum water supply from tanks in any year
tankssupply(TanksProdnYrNd(Yr,N))..
  Qstanks(Yr,N)-sum((YrP,NP)$TanksCapMap(Yr,N,YrP,NP),TanksUnitSize*Qtankscap(YrP,NP)) =l= 0 ;
tankssupply2(TanksProdnYrNd(Yr,N))..
  Qstanks(Yr,N)-sum((YrP,NP)$TanksCapMap(Yr,N,YrP,NP),TanksUnitSize*Qtankscap2(YrP)) =l= 0 ;

* tanksmaxcap upper bound on the total installed tank capacity
tanksmaxcap(TanksCapLastYrNd(Yr,N))..
  sum((YrP,NP)$TanksCapMaxMap(Yr,N,YrP,NP),Qtankscap(YrP,NP)) =l= tanksicap ;
tanksmaxcap2..
  sum( Yr , Qtankscap2(Yr) ) =l= tanksicap ;

* tradesupply(Yr,N) maximum water supply from trade in any year
tradesupply(TradeProdnYrNd(Yr,N))..
  Qstrade(Yr,N)-sum((YrP,NP)$TradeCapMap(Yr,N,YrP,NP),TradeUnitSize*Qtradecap(YrP,NP)) =l= 0 ;
tradesupply2(TradeProdnYrNd(Yr,N))..
  Qstrade(Yr,N)-sum((YrP,NP)$TradeCapMap(Yr,N,YrP,NP),TradeUnitSize*Qtradecap2(YrP)) =l= 0 ;

* trademaxcap upper bound on the total installed trade capacity
trademaxcap(TradeCapLastYrNd(Yr,N))..
  sum((YrP,NP)$TradeCapMaxMap(Yr,N,YrP,NP),Qtradecap(YrP,NP)) =l= tradeicap ;
trademaxcap2..
  sum( Yr , Qtradecap2(Yr) ) =l= tradeicap ;

* aquif2supply(Yr,N) maximum water supply from large aquifers in any year
aquif2supply(aquif2ProdnYrNd(Yr,N))..
  Qsaquif2(Yr,N)-sum((YrP,NP)$aquif2CapMap(Yr,N,YrP,NP),aquif2UnitSize*Qaquif2cap(YrP,NP)) =l= 0 ;
aquif2supply2(aquif2ProdnYrNd(Yr,N))..
  Qsaquif2(Yr,N)-sum((YrP,NP)$aquif2CapMap(Yr,N,YrP,NP),aquif2UnitSize*Qaquif2cap2(YrP)) =l= 0 ;

* aquif2maxcap upper bound on the total installed large aquifer capacity
aquif2maxcap(aquif2CapLastYrNd(Yr,N))..
  sum((YrP,NP)$aquif2CapMaxMap(Yr,N,YrP,NP),Qaquif2cap(YrP,NP)) =l= aquif2icap  ;
aquif2maxcap2..
  sum( Yr , Qaquif2cap2(Yr) ) =l= aquif2icap;


*** Accounting equation to test the levels of demand
Equation Eq_LQd(Yr,N,D)    Eqn calculating level quantity of water demanded;
Variable LQd(Yr,N,D)       Level quantity of water demanded;
Eq_LQd(YrNd(Yr,N),D)..
  LQd(YrNd,D) - sum(L,Qty(YrNd,D,L) * Qd(YrNd,D,L)) =e= 0 ;



*** POLICY CONSTRAINT 1: Water restrictions START WATER RESTRICTIONS
Set  Stage    Restriction stages / St3 /
     Dr(d) restricted sources of demand / D2 / ;
Binary variable
  VRestr(Yr,N,Stage) Binary variable for restrictions of various stages ;
Positive variable
  VRestr0(Yr,N)    Continuous variable to allow for unrestricted demand ;
Equations
  RestCnvx(Yr,N)     Convexity constraint for the binary variables representing water restrictions
  WatRest(Yr,N,D)    Demand constraint resulting from water restrictions
  Trigger(Yr,N)      Trigger levels for water restrictions (one continuous variable) ;
Parameter
  trig(Stage) Minimum water in storage for restr of stage x / st3 = 0 /
  Rest0(D)    Unrestricted demand / D2 = 999 / ;
Scalar trig0  Minimum (was max but think should be min?) water in storage for unrestricted demand / 637 / ;
Table Rest(D,Stage) Water restrictions for each form of demand at each stage
             St3
D2           33     ;

* RestCnvx(Yr,N) Convexity constraint for the binary variables representing water restrictions
RestCnvx(YrNd(Yr,N))..
  sum[ Stage , VRestr(YrNd,Stage)] + VRestr0(YrNd) =e= 1;

*  WatRestWater Demand constraint resulting from water restrictions
WatRest(YrNd(Yr,N),D)$Dr(D)..
  sum[L,Qty(YrNd,D,L) * Qd(YrNd,D,L)] -
  sum[Stage, VRestr(YrNd,Stage) * Rest(D,Stage) ] -
  VRestr0(YrNd) * Rest0(D)
*  - Qstank(yrpt)
  =l= 0;

*  Trigger levels equation: relationship between storage and level of restrictions (one continuous variable)
Trigger(YrNd(Yr,N))..
  sum[Stage, VRestr(YrNd,Stage) * trig(Stage) ] + VRestr0(YrNd) * trig0
  - sum[(YrP,NP)$ChildParent(Yr,N,YrP,NP),Qstdam(YrP,NP)]
  =l= S0$(ord(Yr) eq 1) ;
*** POLICY CONSTRAINT 1: Water restrictions END WATER RESTRICTIONS



*** POLICY CONSTRAINT 2: Long-run marginal cost START LONG-RUN MARGINAL COST
Positive variable
    Equil(D,PrBlk) Variable equilibrating prices within a block ;

Equation
    eq_PriceEquil(Yr,N,D,PrBlk) Equation equilibrating all prices within a given price block ;

* Equation equilibrating all prices within a long-run marginal cost block
eq_PriceEquil(YrNd(Yr,N),D,PrBlk)$[NdPrBlk(Yr,N,PrBlk)]..
sum(L,Qty(YrNd,D,L) * Qd(YrNd,D,L)) =e= Equil(d,PrBlk);
*** POLICY CONSTRAINT 2: Long-run marginal cost END LONG-RUN MARGINAL COST




* Define Market Model 1: the stochastic market model (quantity formulation)

* Bounds
* Upper and lower bounds on dam storage
Qstdam.up(YrNd(Yr,N)) = MaxS0 ;
Qstdam.lo(YrNd(Yr,N)) = MinS0 ;
Qsdesal1.up(YrNd(Yr,N)) = 45 ;
*Qsdesal2.up(YrNd(Yr,N)) = 50 ;
Qsaquif0.up(YrNd(Yr,N)) = 165 ;
Qtankscap.fx(YrNd(YrP,NP)) = 0 ;

model NSWModel
/ Objfn, ConVexdem, demand, damsupply,
  demdamstortp, ConVexQstdam,
*  aquifavgsupply,
  aquifavgsupply1,
  aquifavgsupply2, aquifavgsupply3, aquifavgsupply4,
  desalsupply, desalmaxcap,
  aquifsupply, aquifmaxcap,
  recycsupply, recycmaxcap,
  tankssupply, tanksmaxcap,
  tradesupply, trademaxcap,
  aquif2supply, aquif2maxcap
  , Eq_LQd
*  , RestCnvx, WatRest, Trigger
*  , eq_PriceEquil
/ ;

model NSWModel2
/ Objfn2, ConVexdem, demand, damsupply,
  demdamstortp, ConVexQstdam,
*  aquifavgsupply,
  aquifavgsupply1,
  aquifavgsupply2, aquifavgsupply3, aquifavgsupply4,
  desalsupply2, desalmaxcap2,
  aquifsupply2, aquifmaxcap2,
  recycsupply2, recycmaxcap2,
  tankssupply2, tanksmaxcap2,
  tradesupply2, trademaxcap2,
  aquif2supply2, aquif2maxcap2
  , Eq_LQd
*  , RestCnvx, WatRest, Trigger
*  , eq_PriceEquil
/ ;


model CostModel
/ ObjfnC, demandF, damsupply,
  demdamstortp, ConVexQstdam,
*  dammaxstor,
  aquifavgsupply1,
  aquifavgsupply2, aquifavgsupply3, aquifavgsupply4,
  desalsupply, desalmaxcap,
  aquifsupply, aquifmaxcap,
  recycsupply, recycmaxcap,
  tankssupply, tanksmaxcap,
  tradesupply, trademaxcap,
  aquif2supply, aquif2maxcap
/ ;

model CostModel2
/ ObjfnC2, demandF, damsupply,
  demdamstortp, ConVexQstdam,
*  dammaxstor,
  aquifavgsupply1,
  aquifavgsupply2, aquifavgsupply3, aquifavgsupply4,
  desalsupply2, desalmaxcap2,
  aquifsupply2, aquifmaxcap2,
  recycsupply2, recycmaxcap2,
  tankssupply2, tanksmaxcap2,
  tradesupply2, trademaxcap2,
  aquif2supply2, aquif2maxcap2
/ ;

*  MarketNSW.Workspace=17000 ;
*  CostModel.Optfile=1 ;

*NSWModel.savepoint = 1 ;
* Solve Model 1: the market model (quantity formulation)
*Execute_loadpoint 'marketNSW_p.gdx' ;


* Variable demand, multi-stage
*NSWModel.Optfile=1 ;
*solve NSWModel maximising NSW using mip ;

* Variable demand, two-stage
*NSWModel2.Optfile=1 ;
*solve NSWModel2 maximising NSW using mip ;

* Fixed demand, multi-stage
CostModel.Optfile=1 ;
solve CostModel minimising cost using mip ;

* Fixed demand, two-stage
*CostModel2.Optfile=1 ;
*solve CostModel2 minimising cost using mip ;



