$Title Linearised Urban Water Small
$maxcol 125
$onsymlist
$onsymxref
$oninline
$ontext
*** Linear Urban Water Small ***
Outlined below is a multistage stochastic spatial and temporal programming model of an urban
water system. The model maximises Net Social Welfare (Marshallian Consumer plus producer
surplus).
The model includes investment options for desal, aquifers, recycling, tanks, and trade
$offtext
option mip=gurobi ;
*option lp=minos ;
option decimals = 4 ;
option limcol = 0 ;
option limrow = 0 ;
option solslack = 1 ;
OPTION RESLIM = 100000 ;
OPTION ITERLIM = 100000000 ;
option solprint = off ;
option profile = 1 ;
option sysout=on ;

* Variables and equations constituting the model
Free variables
  NSW expected present value of net social welfare (Marshallian consumer + producer surplus) ;

Positive variables
  Qd(Yr,N,D,L) Linearised demand variable for aggregate demand at each node
  Qddam(Yr,N,M) Linearised variable for water stored in the terminal year T
  Qsdam(Yr,N) Quantity supplied from dam in each node
  Qstdam(Yr,N) Quantity of water stored in dam in each node
  Qsdesal1(Yr,N) Quantity supplied from existing desal (Kwinana) at each node
*  Qsdesal2(Yr,N) Quantity supplied from existing desal (Southern) at each node (present model only)
  Qsaquif0(Yr,N) Quantity supplied from existing aquifers at each node
  Qsdesal(Yr,N) Quantity supplied from desal at each node
  Qsaquif(Yr,N) Quantity supplied from aquifers at each node
  Qsrecyc(Yr,N) Quantity supplied from recycling (groundwater replenishment) at each node
  Qstanks(Yr,N) Quantity supplied from tanks at each node
  Qstrade(Yr,N) Quantity supplied from trade at each node
  Qsaquif2 (Yr,N) Quantity supplied from large aquifers at each node ;

Binary variable
  Qdesalcap(Yr,N) additional desal capacity installed at each node
  Qrecyccap(Yr,N) additional recycling (groundwater replenishment) capacity installed at each node
  Qaquif2cap(Yr,N) additionl large aquifer capacity installed at each node  ;

Positive variables
  Qtankscap(Yr,N) additional tank capacity installed at each node
  Qtradecap(Yr,N) additional trade capacity installed at each node
  Qaquifcap(Yr,N) additional aquifer capacity installed at each node ;

Equations
  Objfn Objective function
  ConVexDem(Yr,N,D) constraint for convexity of linearised demand
  ConVexQstdam(Yr,N) constraint for convexity of terminal storage
  demand(Yr,N) balance of water demanded
  demdamstortp(Yr,N) balance of the demand for dam water storage in the last year
  damsupply(Yr,N) water supply from dams
  dammaxstor(Yr,N) maximum water storage capacity of dams
  aquifavgsupply1(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply2(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply3(Yr,N) average supply from aquifers for each scenario
  aquifavgsupply4(Yr,N) average supply from aquifers for each scenario
  desalsupply(Yr,N) water supply from desal in each node
  desalmaxcap(Yr,N) maximum desal capacity
  aquifsupply(Yr,N) water supply from aquifers in each node
  aquifmaxcap(Yr,N) maximum aquifer capacity
  recycsupply(Yr,N) water supply from recycling in each node
  recycmaxcap(Yr,N) maximum recycling capacity
  tankssupply(Yr,N) water supply from tanks in each node
  tanksmaxcap(Yr,N) maximum tank capacity
  tradesupply(Yr,N) water supply from trade in each node
  trademaxcap(Yr,N) maximum trade capacity
  aquif2supply(Yr,N) water supply from large aquifers in each node
  aquif2maxcap(Yr,N) maximum large aquifer capacity  ;

*  Objfn      Objective function to maximise net social welfare
Objfn..
  nsw =e=
    sum{YrNd(Yr,N),
      sum[(D,L),
        Area(Yr,N,D,L)*Qd(Yr,N,D,L)
      ]

      + sum[M$LastYrNd(Yr,N),
         AreaTSRest(Yr,N,M)*Qddam(Yr,N,M)
      ]

      - [CapExDesal(Yr,N)*Qdesalcap(Yr,N)]$DesalCapInv(Yr,N)
      - [OpExDesal(Yr,N)*Qsdesal(Yr,N)]$DesalProdnYrNd(Yr,N)

      - [CapExAquif(Yr,N)*Qaquifcap(Yr,N)]$AquifCapInv(Yr,N)
      - [OpExAquif(Yr,N)*Qsaquif(Yr,N)]$AquifProdnYrNd(Yr,N)

      - [CapExRecyc(Yr,N)*Qrecyccap(Yr,N)]$RecycCapInv(Yr,N)
      - [OpExRecyc(Yr,N)*Qsrecyc(Yr,N)]$RecycProdnYrNd(Yr,N)

      - [CapExTanks(Yr,N)*Qtankscap(Yr,N)]$TanksCapInv(Yr,N)
      - [OpExTanks(Yr,N)*Qstanks(Yr,N)]$TanksProdnYrNd(Yr,N)

      - [CapExTrade(Yr,N)*Qtradecap(Yr,N)]$TradeCapInv(Yr,N)
      - [OpExTrade(Yr,N)*Qstrade(Yr,N)]$TradeProdnYrNd(Yr,N)

      - [CapExAquif2(Yr,N)*Qaquif2cap(Yr,N)]$Aquif2CapInv(Yr,N)
      - [OpExAquif2(Yr,N)*Qsaquif2(Yr,N)]$Aquif2ProdnYrNd(Yr,N)

      - [OpExDam(Yr,N)*Qsdam(Yr,N)]
      - [OpExDesal1(Yr,N)*Qsdesal1(Yr,N)]
      - [OpExAquif0(Yr,N)*Qsaquif0(Yr,N)]
*      - [OpExDesal2(Yr,N)*Qsdesal2(Yr,N)]$Desal2ProdnYrNd(Yr,N)
    } ;

*  ConVexDem(Yr,N) constraint for convexity of linearised demands
ConVexdem(YrNd(Yr,N),D)..
  sum(L,Qd(Yr,N,D,L)) =l= 1.0 ;

* demand(Yr,N) water demand balance
demand(YrNd(Yr,N))..
  sum[(D,L),Qty(Yr,N,D,L)*Qd(Yr,N,D,L)]
  - qsdam(Yr,N) - qsdesal1(Yr,N)
*  - qsdesal2(Yr,N)$Desal2ProdnYrNd(Yr,N)
  - qsaquif0(Yr,N)
  - qsdesal(Yr,N)$DesalProdnYrNd(Yr,N) - qsaquif(Yr,N)$AquifProdnYrNd(Yr,N)
  - qsrecyc(Yr,N)$RecycProdnYrNd(Yr,N) - qstanks(Yr,N)$TanksProdnYrNd(Yr,N)
  - qstrade(Yr,N)$TradeProdnYrNd(Yr,N)
  - qsaquif2(Yr,N)$aquif2ProdnYrNd(Yr,N)
  =l= 0 ;

* demdamstortp(Yr,N) balance of the demand for dam water storage in the terminal period
demdamstortp(LastYrNd(Yr,N))..
  sum[M,QtyTS(Yr,N,M)*Qddam(Yr,N,M)] - Qstdam(Yr,N) =l= 0 ;

* ConVexQstdam(Yr,N)constraint for convexity of terminal storage
ConVexQstdam(LastYrNd(Yr,N))..
  sum(M,Qddam(Yr,N,M)) =e= 1.0 ;

* damsupply(Yr,N) water supply from dam
damsupply(YrNd(Yr,N))..
  Qsdam(Yr,N) + Qstdam(Yr,N) - sum[(YrP,NP)$ChildParent(Yr,N,YrP,NP),(1-EvapLoss)*Qstdam(YrP,NP)]
  =l= InflowAtNode(Yr,N) ;

* dammaxstor(Yr,N) maximum water storage capacity of dam in any year
dammaxstor(YrNd(Yr,N))..
  Qstdam(Yr,N) =l= MaxS0 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply1(YrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap1(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 5*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply2(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap2(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 4*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply3(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap3(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 3*120 ;

*aquifavgsupply(Yr,N) average water supply from aquifers
aquifavgsupply4(LastYrNd(Yr,N))..
  sum[(Yrp,Np)$AquiAvgMap4(Yr,N,Yrp,Np),Qsaquif0(Yrp,Np)] =l= 2*120 ;

* desalsupply(Yr,N) maximum water supply from desal in any year
desalsupply(DesalProdnYrNd(Yr,N))..
  Qsdesal(Yr,N)-sum((YrP,NP)$DesalCapMap(Yr,N,YrP,NP),DesalUnitSize*Qdesalcap(YrP,NP)) =l= 0 ;

* desalmaxcap upper bound on the total installed desal capacity
desalmaxcap(DesalCapLastYrNd(Yr,N))..
  sum((YrP,NP)$DesalCapMaxMap(Yr,N,YrP,NP),Qdesalcap(YrP,NP)) =l= desalicap ;

* aquifsupply(Yr,N) maximum water supply from aquifers in any year
aquifsupply(AquifProdnYrNd(Yr,N))..
  Qsaquif(Yr,N)-sum((YrP,NP)$AquifCapMap(Yr,N,YrP,NP),AquifUnitSize*Qaquifcap(YrP,NP)) =l= 0 ;

* aquifmaxcap upper bound on the total installed aquifer capacity
aquifmaxcap(AquifCapLastYrNd(Yr,N))..
  sum((YrP,NP)$AquifCapMaxMap(Yr,N,YrP,NP),Qaquifcap(YrP,NP)) =l= aquificap ;

* recycsupply(Yr,N) maximum water supply from recycling in any year
recycsupply(RecycProdnYrNd(Yr,N))..
  Qsrecyc(Yr,N)-sum((YrP,NP)$RecycCapMap(Yr,N,YrP,NP),RecycUnitSize*Qrecyccap(YrP,NP)) =l= 0 ;

* recycmaxcap upper bound on the total installed recyc capacity
recycmaxcap(RecycCapLastYrNd(Yr,N))..
  sum((YrP,NP)$RecycCapMaxMap(Yr,N,YrP,NP),Qrecyccap(YrP,NP)) =l= recycicap ;

* tankssupply(Yr,N) maximum water supply from tanks in any year
tankssupply(TanksProdnYrNd(Yr,N))..
  Qstanks(Yr,N)-sum((YrP,NP)$TanksCapMap(Yr,N,YrP,NP),TanksUnitSize*Qtankscap(YrP,NP)) =l= 0 ;

* tanksmaxcap upper bound on the total installed tank capacity
tanksmaxcap(TanksCapLastYrNd(Yr,N))..
  sum((YrP,NP)$TanksCapMaxMap(Yr,N,YrP,NP),Qtankscap(YrP,NP)) =l= tanksicap ;

* tradesupply(Yr,N) maximum water supply from trade in any year
tradesupply(TradeProdnYrNd(Yr,N))..
  Qstrade(Yr,N)-sum((YrP,NP)$TradeCapMap(Yr,N,YrP,NP),TradeUnitSize*Qtradecap(YrP,NP)) =l= 0 ;

* trademaxcap upper bound on the total installed trade capacity
trademaxcap(TradeCapLastYrNd(Yr,N))..
  sum((YrP,NP)$TradeCapMaxMap(Yr,N,YrP,NP),Qtradecap(YrP,NP)) =l= tradeicap ;

* aquif2supply(Yr,N) maximum water supply from large aquifers in any year
aquif2supply(aquif2ProdnYrNd(Yr,N))..
  Qsaquif2(Yr,N)-sum((YrP,NP)$aquif2CapMap(Yr,N,YrP,NP),aquif2UnitSize*Qaquif2cap(YrP,NP)) =l= 0 ;

* aquif2maxcap upper bound on the total installed large aquifer capacity
aquif2maxcap(aquif2CapLastYrNd(Yr,N))..
  sum((YrP,NP)$aquif2CapMaxMap(Yr,N,YrP,NP),Qaquif2cap(YrP,NP)) =l= aquif2icap ;


*** Accounting equation to test the levels of demand
Equation Eq_LQd(Yr,N,D)    Eqn calculating level quantity of water demanded;
Variable LQd(Yr,N,D)       Level quantity of water demanded;
Eq_LQd(YrNd(Yr,N),D)..
  LQd(YrNd,D) - sum(L,Qty(YrNd,D,L) * Qd(YrNd,D,L)) =e= 0 ;



*** POLICY CONSTRAINT 1: Water restrictions START WATER RESTRICTIONS
Set  Stage    Restriction stages / St3 /
     Dr(d) restricted sources of demand / D2 / ;
Binary variable
  VRestr(Yr,N,Stage) Binary variable for restrictions of various stages ;
Positive variable
  VRestr0(Yr,N)    Continuous variable to allow for unrestricted demand ;
Equations
  RestCnvx(Yr,N)     Convexity constraint for the binary variables representing water restrictions
  WatRest(Yr,N,D)    Demand constraint resulting from water restrictions
  Trigger(Yr,N)      Trigger levels for water restrictions (one continuous variable) ;
Parameter
  trig(Stage) Minimum water in storage for restr of stage x / st3 = 0 /
  Rest0(D)    Unrestricted demand / D2 = 999 / ;
Scalar trig0  Minimum (was max but think should be min?) water in storage for unrestricted demand / 155.5 / ;
Table Rest(D,Stage) Water restrictions for each form of demand at each stage
             St3
D2           66.6     ;

* RestCnvx(Yr,N) Convexity constraint for the binary variables representing water restrictions
RestCnvx(YrNd(Yr,N))..
  sum[ Stage , VRestr(YrNd,Stage)] + VRestr0(YrNd) =e= 1;

*  WatRestWater Demand constraint resulting from water restrictions
WatRest(YrNd(Yr,N),D)$Dr(D)..
  sum[L,Qty(YrNd,D,L) * Qd(YrNd,D,L)] -
  sum[Stage, VRestr(YrNd,Stage) * Rest(D,Stage) ] -
  VRestr0(YrNd) * Rest0(D)
  - Qstanks(yrnd)
  =l= 0;

*  Trigger levels equation: relationship between storage and level of restrictions (one continuous variable)
Trigger(YrNd(Yr,N))..
  sum[Stage, VRestr(YrNd,Stage) * trig(Stage) ] + VRestr0(YrNd) * trig0
  - sum[(YrP,NP)$ChildParent(Yr,N,YrP,NP),Qstdam(YrP,NP)]
  =l= S0$(ord(Yr) eq 1) ;
*** POLICY CONSTRAINT 1: Water restrictions END WATER RESTRICTIONS




* Define Market Model 1: the stochastic market model (quantity formulation)

* Bounds
* Upper and lower bounds on dam storage, desal supply, aquifer supply and tank capacity
Qstdam.up(YrNd(Yr,N)) = MaxS0 ;
Qstdam.lo(YrNd(Yr,N)) = MinS0 ;
Qsdesal1.up(YrNd(Yr,N)) = 45 ;
*Qsdesal2.up(YrNd(Yr,N)) = 50 ;
Qsaquif0.up(YrNd(Yr,N)) = 165 ;
Qtankscap.fx(YrNd(YrP,NP)) = 0 ;

model MarketNSW
/ Objfn, ConVexdem, demand, damsupply,
  demdamstortp, ConVexQstdam,
  aquifavgsupply1,
  aquifavgsupply2, aquifavgsupply3,
  aquifavgsupply4,
  desalsupply, desalmaxcap,
  aquifsupply, aquifmaxcap,
  recycsupply, recycmaxcap,
  tankssupply, tanksmaxcap,
  tradesupply, trademaxcap
  aquif2supply, aquif2maxcap
  , RestCnvx, WatRest, Trigger
*  , Eq_LQd
/ ;
*  MarketNSW.Workspace=17000 ;
  MarketNSW.Optfile=1 ;
*  MarketNSW.savepoint = 1 ;
* Solve Model 1: the market model (quantity formulation)
Execute_loadpoint 'marketNSW_p.gdx' ;
*option optcr = 0.0001 ;
solve MarketNSW maximizing NSW using mip ;

